// Generated by dts-bundle-generator v9.5.1

/**
 * **Defines a modification operation to be applied to JSON data**
 *
 * Represents a single transformation instruction
 * @example
 * ```ts
 * const operation: Operation = {
 *   path: ["users", 0, "name"],
 *   action: "set",
 *   value: "John Smith"
 * }
 * ```
 */
export type Operation = {
	path: (string | number)[];
} & ({
	action: "set" | "append" | "prepend";
	value: unknown;
} | {
	action: "delete";
	value?: unknown;
});
/**
 * **Details about a failed operation attempt**
 *
 * Contains complete operation context for debugging
 * @example
 * ```ts
 * {
 *   index: 0,
 *   operation: {path: ["invalid"], action: "set", value: "test"},
 *   reason: "Target path does not exist"
 * }
 * ```
 */
export type OperationError = {
	index: number;
	operation: Operation;
	reason: string;
};
/**
 * **Configuration options for JsonMason behavior**
 * @example
 * ```ts
 * const config: Config = {
 *   strict: false // Continue on errors
 * }
 * ```
 */
export type Config = {
	/** When true, throws errors immediately. Default: true */
	strict?: boolean;
};
/**
 * **Static utility methods for JSON operations**
 *
 * Provides core functionality for data manipulation
 * @example
 * ```ts
 * const value = JsonMasonUtils.get(obj, ["users", 0]);
 * const newObj = JsonMasonUtils.set(obj, ["count"], 42);
 * ```
 */
export declare class JsonMasonUtils {
	/**
	 * **Creates a deep clone of an object**
	 *
	 * Uses structured clone for maximum compatibility
	 * @example
	 * ```ts
	 * const original = {nested: {value: 1}};
	 * const clone = JsonMasonUtils.clone(original);
	 * clone.nested.value = 2; // Doesn't affect original
	 * ```
	 */
	static clone<T>(source: T): T;
	/**
	 * **Safely retrieves a nested value from an object**
	 * @example
	 * ```ts
	 * const obj = {users: [{name: "John"}]};
	 * const name = JsonMasonUtils.get(obj, ["users", 0, "name"]); // "John"
	 * const missing = JsonMasonUtils.get(obj, ["invalid"]); // undefined
	 * ```
	 */
	static get<T>(obj: T, path: (string | number)[]): unknown;
	/**
	 * **Sets a value at a nested path**
	 *
	 * Creates intermediate objects/arrays as needed
	 * @example
	 * ```ts
	 * const obj = {users: []};
	 * const result = JsonMasonUtils.set(obj, ["users", 0, "name"], "John");
	 * // result = {users: [{name: "John"}]}
	 * ```
	 */
	static set<T>(obj: T, path: (string | number)[], value: unknown): T;
	/**
	 * **Validates if a value is an array**
	 *
	 * Centralizes array checking logic
	 * @example
	 * ```ts
	 * const arr = [1, 2, 3];
	 * JsonMasonUtils.validateArray(arr, ["numbers"]); // OK
	 * JsonMasonUtils.validateArray({}, ["invalid"]); // Throws Error
	 * ```
	 */
	static validateArray(value: unknown, path: (string | number)[]): asserts value is unknown[];
	/**
	 * **Validates if a value is a string**
	 *
	 * Centralizes string validation logic
	 * @example
	 * ```ts
	 * const str = "test";
	 * JsonMasonUtils.validateString(str, ["path"]); // OK
	 * JsonMasonUtils.validateString(123, ["invalid"]); // Throws Error
	 * ```
	 */
	static validateString(value: unknown, path: (string | number)[]): asserts value is string;
	/**
	 * **Validates if value is array or string**
	 *
	 * Ensures target is appendable/prependable
	 * @example
	 * ```ts
	 * const arr = [1, 2, 3];
	 * const str = "test";
	 * JsonMasonUtils.validateAppendable(arr, ["array"]); // OK
	 * JsonMasonUtils.validateAppendable(str, ["string"]); // OK
	 * JsonMasonUtils.validateAppendable({}, ["invalid"]); // Throws Error
	 * ```
	 */
	static validateAppendable(value: unknown, path: (string | number)[]): asserts value is unknown[] | string;
}
/**
 * **Core class for performing JSON transformations**
 *
 * Handles operations like setting values and array modifications
 * @example
 * ```ts
 * const mason = new JsonMason();
 * const result = mason.apply(data, [
 *   {path: ["name"], action: "set", value: "John"},
 *   {path: ["tags"], action: "append", value: "new"}
 * ]);
 * ```
 */
export declare class JsonMason {
	private errors;
	/**
	 * **Retrieves errors from the last operation**
	 *
	 * Only populated when strict mode is disabled
	 * @example
	 * ```ts
	 * const mason = new JsonMason();
	 * mason.apply(data, operations, {strict: false});
	 * const errors = mason.getErrors();
	 * // [{index: 0, operation: {...}, reason: "..."}]
	 * ```
	 */
	getErrors(): OperationError[];
	/**
	 * **Applies multiple operations to a source object**
	 *
	 * Processes operations sequentially with error handling
	 * @example
	 * ```ts
	 * const source = {count: 0, items: []};
	 * const operations = [
	 *   {path: ["count"], action: "set", value: 1},
	 *   {path: ["items"], action: "append", value: "first"}
	 * ];
	 * const result = mason.apply(source, operations);
	 * // {count: 1, items: ["first"]}
	 * ```
	 */
	apply<T>(source: T, operations: Operation[], config?: Config): T;
	private executeOperation;
}

export {};
