// src/index.ts
class JsonMasonUtils {
  static clone(source) {
    return structuredClone(source);
  }
  static get(obj, path) {
    let current = obj;
    for (const key of path) {
      if (current == null)
        return;
      current = current[key];
    }
    return current;
  }
  static set(obj, path, value) {
    if (path.length === 0)
      return value;
    const result = this.clone(obj);
    let current = result;
    for (let i = 0;i < path.length - 1; i++) {
      const key = path[i];
      const nextKey = path[i + 1];
      if (!(key in current)) {
        current[key] = typeof nextKey === "number" ? [] : {};
      }
      current = current[key];
    }
    current[path[path.length - 1]] = value;
    return result;
  }
  static validateArray(value, path) {
    if (!Array.isArray(value)) {
      throw new Error(`Target at path ${path.join(".")} is not an array`);
    }
  }
  static validateString(value, path) {
    if (typeof value !== "string") {
      throw new Error(`Target at path ${path.join(".")} is not a string`);
    }
  }
  static validateAppendable(value, path) {
    if (!Array.isArray(value) && typeof value !== "string") {
      throw new Error(`Target at path ${path.join(".")} is not an array or string`);
    }
  }
}

class JsonMason {
  errors = [];
  getErrors() {
    return this.errors;
  }
  apply(source, operations, config) {
    const strict = config?.strict ?? true;
    this.errors = [];
    let result = JsonMasonUtils.clone(source);
    for (let i = 0;i < operations.length; i++) {
      const operation = operations[i];
      try {
        result = this.executeOperation(result, operation);
      } catch (error) {
        const operationError = {
          index: i,
          operation,
          reason: error instanceof Error ? error.message : String(error)
        };
        if (strict)
          throw new Error(operationError.reason);
        this.errors.push(operationError);
        return source;
      }
    }
    return result;
  }
  executeOperation(source, operation) {
    const { path, action, value } = operation;
    switch (action) {
      case "set":
        return JsonMasonUtils.set(source, path, value);
      case "delete": {
        if (path.length === 0) {
          throw new Error("Cannot delete root object");
        }
        const result = JsonMasonUtils.clone(source);
        const parentPath = path.slice(0, -1);
        const target = parentPath.length === 0 ? result : JsonMasonUtils.get(result, parentPath);
        if (target == null) {
          throw new Error(`Parent path ${parentPath.join(".")} does not exist`);
        }
        const key = path[path.length - 1];
        if (Array.isArray(target)) {
          if (typeof key === "number") {
            target.splice(key, 1);
          } else {
            throw new Error(`Invalid array index: ${key}`);
          }
        } else if (typeof target === "object") {
          delete target[key];
        } else {
          throw new Error(`Cannot delete from non-object at ${parentPath.join(".")}`);
        }
        return result;
      }
      case "append":
      case "prepend": {
        const current = JsonMasonUtils.get(source, path);
        JsonMasonUtils.validateAppendable(current, path);
        let newValue;
        if (typeof current === "string") {
          if (typeof value !== "string") {
            throw new Error("Can only append/prepend strings to strings");
          }
          newValue = action === "append" ? current + value : value + current;
        } else {
          newValue = action === "append" ? [...current, value] : [value, ...current];
        }
        return JsonMasonUtils.set(source, path, newValue);
      }
      default:
        throw new Error(`Unknown operation: ${action}`);
    }
  }
}
export {
  JsonMasonUtils,
  JsonMason
};
